#ifndef MYX_BASE_ENUM_BITWISE_OPERATIONS_HPP_
#define MYX_BASE_ENUM_BITWISE_OPERATIONS_HPP_


#pragma once

#include <type_traits>

namespace myx {

namespace base {

template< typename Enum >
struct EnableBitMaskOperators
{
	static const bool enable = false;
};

template< typename Enum >
typename std::enable_if< EnableBitMaskOperators< Enum >::enable, Enum >::type
operator |( Enum lhs, Enum rhs )
{
	using underlying = typename std::underlying_type< Enum >::type;
	return( static_cast< Enum >(
			static_cast< underlying >( lhs ) |
			static_cast< underlying >( rhs )
			) );
}

} // namespace base

} // namespace myx

/**
 * @brief Макрос, предоставляющий возможность выполнять битовые операции в enum class
 *
 *  Источник: http://blog.bitwigglers.org/using-enum-classes-as-type-safe-bitmasks/
 *  Пример использования:
 *
 *  namespace ns {
 *  enum class Permissions
 *  {
 *     Readable   = 0x4,
 *     Writeable  = 0x2,
 *     Executable = 0x1
 *  };
 *  }
 *  ENABLE_BITMASK_OPERATORS(ns::Permissions)
 */

#define ENABLE_BITMASK_OPERATORS( x ) \
	template<> \
	struct myx::base::EnableBitMaskOperators< x > \
	{ \
		static const bool enable = true; \
	};

#endif // ifndef MYX_BASE_ENUM_BITWISE_OPERATIONS_HPP_
