#ifndef MYX_REDIS_REQUEST_HPP_
#define MYX_REDIS_REQUEST_HPP_

#include <QObject>
#include <QScopedPointer>

#include <config.hpp>
#include <reply.hpp>

namespace myx {

namespace redis {

class MYX_REDIS_EXPORT RequestPrivate;

/**
 * @brief Represents a Redis command and its response
 */
class MYX_REDIS_EXPORT Request : public QObject
{
	Q_OBJECT

public:

	/**
	 * @brief Initializes the request
	 * @param parent the parent QObject
	 */
	explicit Request( QObject* parent = nullptr );

	/**
	 * @brief Destroys the request
	 */
	virtual ~Request() = default;

	/**
	 * @brief Waits for the reply to be received
	 * @param msecs the amount of time in milliseconds to wait
	 * @return true if the reply was received
	 */
	bool waitForReply( int msecs = 10000 );

	/**
	 * @brief Emitted when a reply is received
	 * @param reply the reply received
	 */
	Q_SIGNAL void reply( const myx::redis::Reply& reply );

private:

	const QScopedPointer< RequestPrivate > d;
}; // class MYX_REDIS_EXPORT

} // namespace redis

} // namespace myx

#endif // MYX_REDIS_REQUEST_HPP_
